/*************************************************************************
 *
 *  OpenOffice.org - a multi-platform office productivity suite
 *
 *  $RCSfile: sc_htmlexp2.cxx,v $
 *
 *  $Revision: 1.4 $
 *
 *  last change: $Author: rt $ $Date: 2005/09/07 17:36:20 $
 *
 *  The Contents of this file are made available subject to
 *  the terms of GNU Lesser General Public License Version 2.1.
 *
 *
 *    GNU Lesser General Public License Version 2.1
 *    =============================================
 *    Copyright 2005 by Sun Microsystems, Inc.
 *    901 San Antonio Road, Palo Alto, CA 94303, USA
 *
 *    This library is free software; you can redistribute it and/or
 *    modify it under the terms of the GNU Lesser General Public
 *    License version 2.1, as published by the Free Software Foundation.
 *
 *    This library is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *    Lesser General Public License for more details.
 *
 *    You should have received a copy of the GNU Lesser General Public
 *    License along with this library; if not, write to the Free Software
 *    Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *    MA  02111-1307  USA
 *
 ************************************************************************/

#ifdef PCH
#include "filt_pch.hxx"
#endif

#pragma hdrstop

//------------------------------------------------------------------------

#ifdef MAC
#define _SYSDEP_HXX
#endif

#include <bf_svx/svditer.hxx>
#include <bf_svx/svdograf.hxx>
#include <bf_svx/svdoole2.hxx>
#include <bf_svx/svdpage.hxx>
#include <bf_svx/xoutbmp.hxx>
#ifndef _SVDXCGV_HXX
#include <bf_svx/svdxcgv.hxx>
#endif
#include <so3/ipobj.hxx>
#include <sot/exchange.hxx>
#include <svtools/htmlkywd.hxx>
#include <svtools/htmlout.hxx>
#include <svtools/transfer.hxx>
#ifndef SVTOOLS_URIHELPER_HXX
#include <svtools/urihelper.hxx>
#endif
#include <tools/urlobj.hxx>

#if defined(WIN) || defined(WNT)
#ifndef _SVWIN_H
#include <tools/svwin.h>
#endif
#endif

#include "htmlexp.hxx"
#include "global.hxx"
#include "flttools.hxx"
#include "document.hxx"
#include "drwlayer.hxx"
#include "so3/staticbaseurl.hxx"
namespace binfilter {


//------------------------------------------------------------------------

void ScHTMLExport::PrepareGraphics( ScDrawLayer* pDrawLayer, USHORT nTab,
		USHORT nStartCol, USHORT nStartRow,	USHORT nEndCol, USHORT nEndRow )
{
	if ( pDrawLayer->HasObjectsInRows( nTab, nStartRow, nEndRow ) )
	{
		SdrPage* pDrawPage = pDrawLayer->GetPage( nTab );
		if ( pDrawPage )
		{
			bTabHasGraphics = TRUE;
			FillGraphList( pDrawPage, nTab,
				nStartCol, nStartRow, nEndCol, nEndRow );
			for ( ScHTMLGraphEntry* pE = aGraphList.First(); pE;
					pE = aGraphList.Next() )
			{
				if ( !pE->bInCell )
				{	// nicht alle in Zellen: einige neben Tabelle
					bTabAlignedLeft = TRUE;
					break;
				}
			}
		}
	}
}


void ScHTMLExport::FillGraphList( const SdrPage* pPage, USHORT nTab,
		USHORT nStartCol, USHORT nStartRow,	USHORT nEndCol, USHORT nEndRow )
{
	ULONG	nObjCount = pPage->GetObjCount();
	if ( nObjCount )
	{
		Rectangle aRect;
		if ( !bAll )
			aRect = pDoc->GetMMRect( nStartCol, nStartRow, nEndCol, nEndRow, nTab );
		SdrObjListIter aIter( *pPage, IM_FLAT );
		SdrObject* pObject = aIter.Next();
		while ( pObject )
		{
			Rectangle aObjRect = pObject->GetBoundRect();
			if ( bAll || aRect.IsInside( aObjRect ) )
			{
                Size aSpace;
                ScRange aR = pDoc->GetRange( nTab, aObjRect );
                // Rectangle in mm/100
                Size aSize( MMToPixel( aObjRect.GetSize() ) );
                // If the image is somewhere in a merged range we must
                // move the anchor to the upper left (THE span cell).
                pDoc->ExtendOverlapped( aR );
                USHORT nCol1 = aR.aStart.Col();
                USHORT nRow1 = aR.aStart.Row();
                USHORT nCol2 = aR.aEnd.Col();
                USHORT nRow2 = aR.aEnd.Row();
                // All cells empty under object?
                BOOL bInCell = (pDoc->GetEmptyLinesInBlock(
                    nCol1, nRow1, nTab, nCol2, nRow2, nTab, DIR_TOP )
                    == (nRow2 - nRow1));    // rows-1 !
                if ( bInCell )
                {   // Spacing in spanning cell
                    Rectangle aCellRect = pDoc->GetMMRect(
                        nCol1, nRow1, nCol2, nRow2, nTab );
                    aSpace = MMToPixel( Size(
                        aCellRect.GetWidth() - aObjRect.GetWidth(),
                        aCellRect.GetHeight() - aObjRect.GetHeight() ));
                    aSpace.Width() += (nCol2-nCol1) * (nCellSpacing+1);
                    aSpace.Height() += (nRow2-nRow1) * (nCellSpacing+1);
                    aSpace.Width() /= 2;
                    aSpace.Height() /= 2;
                }
                ScHTMLGraphEntry* pE = new ScHTMLGraphEntry( pObject,
                    aR, aSize, bInCell, aSpace );
                aGraphList.Insert( pE, LIST_APPEND );
			}
			pObject = aIter.Next();
		}
	}
}


void ScHTMLExport::WriteGraphEntry( ScHTMLGraphEntry* pE )
{
	SdrObject* pObject = pE->pObject;
	ByteString aOpt;
	(((aOpt += ' ') += sHTML_O_width) += '=') +=
		ByteString::CreateFromInt32( pE->aSize.Width() );
	(((aOpt += ' ') += sHTML_O_height) += '=') +=
		ByteString::CreateFromInt32( pE->aSize.Height() );
	if ( pE->bInCell )
	{
		(((aOpt += ' ') += sHTML_O_hspace) += '=') +=
			ByteString::CreateFromInt32( pE->aSpace.Width() );
		(((aOpt += ' ') += sHTML_O_vspace) += '=') +=
			ByteString::CreateFromInt32( pE->aSpace.Height() );
	}
	switch ( pObject->GetObjIdentifier() )
	{
		case OBJ_GRAF:
		{
			const SdrGrafObj* pSGO = (SdrGrafObj*)pObject;
			const SdrGrafObjGeoData* pGeo = (SdrGrafObjGeoData*)pSGO->GetGeoData();
			USHORT nMirrorCase = (pGeo->aGeo.nDrehWink == 18000 ?
					( pGeo->bMirrored ? 3 : 4 ) : ( pGeo->bMirrored ? 2 : 1 ));
			BOOL bHMirr = ( ( nMirrorCase == 2 ) || ( nMirrorCase == 4 ) );
			BOOL bVMirr = ( ( nMirrorCase == 3 ) || ( nMirrorCase == 4 ) );
			ULONG nXOutFlags = 0;
			if ( bHMirr )
				nXOutFlags |= XOUTBMP_MIRROR_HORZ;
			if ( bVMirr )
				nXOutFlags |= XOUTBMP_MIRROR_VERT;
			String aLinkName;
			if ( pSGO->IsLinkedGraphic() )
				aLinkName = pSGO->GetFileName();
			WriteImage( aLinkName, pSGO->GetGraphic(), aOpt, nXOutFlags );
			pE->bWritten = TRUE;
		}
		break;
		case OBJ_OLE2:
		{
			const SvInPlaceObjectRef& rRef = ((SdrOle2Obj*)pObject)->GetObjRef();
            DBG_ASSERT( rRef.Is(), "WriteGraphEntry: no OLE ObjRef" );
            if ( rRef.Is() )
            {
                TransferableDataHelper aOleData( rRef->CreateTransferableSnapshot() );
                GDIMetaFile aMtf;
                if( aOleData.GetGDIMetaFile( FORMAT_GDIMETAFILE, aMtf ) )
                {
                    Graphic aGraph( aMtf );
                    String aLinkName;
                    WriteImage( aLinkName, aGraph, aOpt );
                    pE->bWritten = TRUE;
                }
            }
		}
		break;
        default:
        {
            Graphic aGraph( SdrExchangeView::GetObjGraphic(
                pDoc->GetDrawLayer(), pObject ) );
            String aLinkName;
            WriteImage( aLinkName, aGraph, aOpt );
            pE->bWritten = TRUE;
        }
	}
}


void ScHTMLExport::WriteImage( String& rLinkName, const Graphic& rGrf,
			const ByteString& rImgOptions, ULONG nXOutFlags )
{
	// embeddete Grafik -> via WriteGraphic schreiben
	if( !rLinkName.Len() )
	{
		if( aStreamPath.Len() > 0 )
		{
			// Grafik als (JPG-)File speichern
			String aGrfNm( aStreamPath );
            nXOutFlags |= XOUTBMP_USE_NATIVE_IF_POSSIBLE;
            USHORT nErr = XOutBitmap::WriteGraphic( rGrf, aGrfNm,
                _STRINGCONST( "JPG" ), nXOutFlags );
			if( !nErr )		// sonst fehlerhaft, da ist nichts auszugeben
			{
				rLinkName = so3::StaticBaseUrl::SmartRelToAbs( aGrfNm );
				if ( HasCId() )
					MakeCIdURL( rLinkName );
			}
		}
	}
	else
	{
		if( bCopyLocalFileToINet || HasCId() )
		{
			CopyLocalFileToINet( rLinkName, aStreamPath );
			if ( HasCId() )
				MakeCIdURL( rLinkName );
		}
		else
			rLinkName = so3::StaticBaseUrl::SmartRelToAbs( rLinkName );
	}
	if( rLinkName.Len() )
	{	// <IMG SRC="..."[ rImgOptions]>
		rStrm << '<' << sHTML_image << ' ' << sHTML_O_src << "=\"";
		HTMLOutFuncs::Out_String( rStrm, so3::StaticBaseUrl::AbsToRel( rLinkName ), eDestEnc ) << '\"';
		if ( rImgOptions.Len() )
			rStrm << rImgOptions.GetBuffer();
		rStrm << '>' << sNewLine << GetIndentStr();
	}
}




}
